/******************************************************************************
 * $Id: gfx_stream.cc,v 1.6 2006/07/23 02:04:39 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Streams.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 ******************************************************************************/

#define GFX_STREAM_CC 1
#include "gfx.hh"
#include "gfx_stream.hh"
#include "gfx_file.hh"
namespace gfx {

/*****************************************************************************
 * StreambufFile2 constructor.
 * @param	file1, file2
 *          Both required.
 * @param	close1, close2
 *			If destructor should close file.
 *****************************************************************************/
StreambufFile2::StreambufFile2( FILE* file1, FILE* file2, bool close1, bool close2 )
{
	mFile[0]  = file1;
	mFile[1]  = file2;
	mClose[0] = close1;
	mClose[1] = close2;
}

/*****************************************************************************
 * StreambufFile2 destructor.
 *****************************************************************************/
StreambufFile2::~StreambufFile2()
{
	if ( mClose[0] ) fclose( mFile[0] );
	if ( mClose[1] ) fclose( mFile[1] );
}

/*****************************************************************************
 * Either is called by iostream to write out buffer.  xsputn() is optional/optimal.
 *****************************************************************************/
int
StreambufFile2::overflow( int c )
{
	fputc( c, mFile[0] );
	fputc( c, mFile[1] );
	return ~EOF;
}
std::streamsize
StreambufFile2::xsputn( const char* buf, std::streamsize n )
{
	fwrite( buf, n, 1, mFile[0] );
	fwrite( buf, n, 1, mFile[1] );
	return n;
}

} // namespace gfx
