/*****************************************************************************
 * $Id: basic_types.cc,v 1.2 2006/11/08 01:05:33 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Test of basic types in gfx_types.hh.
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "gfx_types.hh"

/*****************************************************************************
 * Test Array class.
 *****************************************************************************/
bool
TestArray( void )
{
	cerr << "============================================\n";
	cerr << "Testing Array class. \n";
	cerr << "============================================\n";

 {
	cerr << "----------- basic test Array<3> --------- \n";
	Array<3> a;
	a[0] = 1;
	a[1] = 2;
	a[2] = 3;
	if (    a[0] != 1
         || a[1] != 2
         || a[2] != 3 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<3,float> --------- \n";
	Array<2,float> a;
	a[0] = 12.34;
	a[1] = 56.78;

	if (    FNE( a[0], 12.34 )
         || FNE( a[1], 56.78 ) ) goto Error;

	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<>(val) constructor> --------- \n";
	Array<2> a(123);		// initialize every element with same value
	if (   a[0] != 123
        || a[1] != 123 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<>() = { } constructor> --------- \n";
	int arr[3] = { 64, 32, 16 };
	Array<3> a = arr;
	if (   a[0] != 64
        || a[1] != 32
        || a[2] != 16 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<> copy constructor> --------- \n";
	Array<2> a1;
	a1[0] = 101;
	a1[1] = 202;
	Array<2> a2 = a1;
	if (    a2[0] != 101
         || a2[1] != 202 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<> operator=() --------- \n";
	Array<2> a1(0);
	Array<2> a2(1);
	a1[0] = 112;
	a1[1] = 221;
	a2 = a1;
	if (    a2[0] != 112
         || a2[1] != 221 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<> operator==/!=()  --------- \n";
	Array<2> a1(3);
	Array<2> a2(4);

	a1 = a2;			// they're equal
	if ( ! (a1 == a2) ) goto Error;
	if ( a1 != a2 )  goto Error;

	a1[0] = 99; a2[0] = 100;	// now they're different
	if ( a1 == a2 ) goto Error;
	if ( ! (a1 != a2) ) goto Error;

	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<>.Fill(),Cnt()  --------- \n";
	Array<50> a(555);
	a.Fill(777);

	if ( a.GetCnt() != 50 ) goto Error;
	for ( uint i = 0; i < a.GetCnt(); ++i )
		if ( a[i] != 777 ) goto Error;

	cerr << "sizeof( Array<50> ) = " << sizeof(a) << endl;

	cerr << "ok \n";
 }

 {
	cerr << "----------- test Array<>.operator<()  --------- \n";
	Array<4> a1(555), a2(555);
	++a2[3];	// slightly larger
	if ( ! (a1 < a2) ) goto Error;
	if (   (a2 < a1) ) goto Error;

	a2[3] = a1[3];
	++a1[0];	// slightly larger
	if (   (a1 < a2 ) ) goto Error;
	if ( ! (a2 < a1 ) ) goto Error;

	a1 = a2;	// equal
	if (   (a1 < a2 ) ) goto Error;
	if (   (a2 < a1 ) ) goto Error;

	cerr << "ok \n";
 }

	return true;
Error:
	cerr << "** FAILED ** \n";
	return false;
}

/*****************************************************************************
 * Test Vertex* class.
 *****************************************************************************/
bool
TestVertex( void )
{
	cerr << "============================================\n";
	cerr << "Testing Vertex* class. \n";
	cerr << "============================================\n";

	// Test operator+()
 {
	WorldVertex v1, v2;
	v1.x = 10.0;
	v1.y = 20.0;
	v1.z = 30.0;
	v2.x = 1.0;
	v2.y = 2.0;
	v2.z = 3.0;
	v1 = v1 + v2;
	if (    int(v1.x) != 11 || int(v1.y) != 22 || int(v1.z) != 33
         || int(v2.x) != 1  || int(v2.y) != 2  || int(v2.z) != 3)
		goto Error;
 }

	// Test operator+=()
 {
	WorldVertex v1, v2;
	v1.x = 10.0;
	v1.y = 20.0;
	v1.z = 30.0;
	v2.x = 1.0;
	v2.y = 2.0;
	v2.z = 3.0;
	v1 += v2;
	if (    int(v1.x) != 11 || int(v1.y) != 22 || int(v1.z) != 33
         || int(v2.x) != 1  || int(v2.y) != 2  || int(v2.z) != 3)
		goto Error;
 }

	// Test operator-()
 {
	WorldVertex v1, v2;
	v1.x = 10.0;
	v1.y = 20.0;
	v1.z = 30.0;
	v2.x = 1.0;
	v2.y = 2.0;
	v2.z = 3.0;

	v1 = v1 - v2;
	if (    int(v1.x) != 10-1  || int(v1.y) != 20-2 || int(v1.z) != 30-3
         || int(v2.x) != 1  || int(v2.y) != 2  || int(v2.z) != 3)
		goto Error;
 }

	// Test operator-=()
 {
	WorldVertex v1, v2;
	v1.x = 10.0;
	v1.y = 20.0;
	v1.z = 30.0;
	v2.x = 1.0;
	v2.y = 2.0;
	v2.z = 3.0;

	v1 -= v2;
	if (    int(v1.x) != 10-1  || int(v1.y) != 20-2 || int(v1.z) != 30-3
         || int(v2.x) != 1  || int(v2.y) != 2  || int(v2.z) != 3)
		goto Error;
 }

	// Test operator*()
 {
	WorldVertex v1, v2;
	v1.x = 10.0;
	v1.y = 20.0;
	v1.z = 30.0;
	v2.x = 1.0;
	v2.y = 2.0;
	v2.z = 3.0;
	v1 = v1 * v2;
	if (    int(v1.x) != 10*1  || int(v1.y) != 20*2 || int(v1.z) != 30*3
         || int(v2.x) != 1  || int(v2.y) != 2  || int(v2.z) != 3)
		goto Error;
 }

	// Test operator*=()
 {
	WorldVertex v1, v2;
	v1.x = 10.0;
	v1.y = 20.0;
	v1.z = 30.0;
	v2.x = 1.0;
	v2.y = 2.0;
	v2.z = 3.0;
	v1 *= v2;
	if (    int(v1.x) != 10*1  || int(v1.y) != 20*2 || int(v1.z) != 30*3
         || int(v2.x) != 1  || int(v2.y) != 2  || int(v2.z) != 3)
		goto Error;
 }

	return true;
Error:
	cerr << "** FAILED ** \n";
	return false;
}

/*****************************************************************************
 * Test Posrad.
 *****************************************************************************/
bool
TestPosrad( void )
{
	cerr << "============================================\n";
	cerr << "Testing Posrad classes. \n";
	cerr << "============================================\n";

 {
	cerr << "----------- test operator[] --------- \n";
	Posrad posrad1( 10, 20, 30, 40 );
	cerr << "Should print 10,20,30,40 \n";
	cerr << posrad1.x << ',' << posrad1.y << ',' << posrad1.z << ',' << posrad1.mRadius << endl;
	if (    posrad1.x != 10
         || posrad1.y != 20
         || posrad1.z != 30
         || posrad1.mRadius != 40 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- different --------- \n";
	Posrad posrad1( 1, 2, 3, 4 );
	Posrad posrad2( 5, 6, 7, 8 );
	cerr << "posrad1 = " << posrad1 << endl;
	cerr << "posrad2 = " << posrad2 << endl;
	cerr << "posrad1 == posrad2 " << (posrad1==posrad2) << endl;
	cerr << "posrad1 != posrad2 " << (posrad1!=posrad2) << endl;
	bool test1 = ! (posrad1 == posrad2);	// should be true if operators work ok
	bool test2 =   (posrad1 != posrad2);
	if ( ! test1 || ! test2 ) goto Error;
	cerr << "ok \n";
 }

 {
	cerr << "----------- copy constructor (same) --------- \n";
	Posrad posrad1( 1, 2, 3, 4 );
	Posrad posrad2 = posrad1;
	cerr << "posrad1 = " << posrad1 << endl;
	cerr << "posrad2 = " << posrad2 << endl;
	cerr << "posrad1 != posrad2 " << (posrad1==posrad2) << endl;
	bool test1 =   (posrad1 == posrad2);	// should be true if operators work ok
	bool test2 = ! (posrad1 != posrad2);
	if ( ! test1 || ! test2 ) goto Error;
	cerr << "ok \n";
 }

	return true;
Error:
	cerr << "** FAILED ** \n";
	return false;
}

/*****************************************************************************
 * Test Rect.
 *****************************************************************************/
bool
TestRect( void )
{
	cerr << "============================================\n";
	cerr << "Testing Rect class. \n";
	cerr << "============================================\n";

	// The Rect class, for speed, assumes (x1,y1) is less than (x2,y2).

 {
	cerr << "----------- testing IfInside() --------- \n";

	Rect<int> rect( 10, 20, 500, 600 );

	if ( ! rect.IfInside( 10, 20 ) ) goto Error;	// barely inside
	if ( ! rect.IfInside( 500, 600 ) ) goto Error;	// barely inside

	if ( ! rect.IfInside( 11, 21 ) ) goto Error;
	if ( ! rect.IfInside( 499, 599 ) ) goto Error;

	if ( rect.IfInside( 0, 0 ) ) goto Error;		// not inside
	if ( rect.IfInside( -10, -20 ) ) goto Error;	// not inside
	if ( rect.IfInside( -510, -620 ) ) goto Error;	// not inside
	if ( rect.IfInside( 1000, 1000 ) ) goto Error;	// not inside
	if ( rect.IfInside( 10, 601 ) ) goto Error;		// not inside

	cerr << "ok\n";
 }

 {
	cerr << "----------- testing IfOutside() --------- \n";

	Rect<int> rect( 200, 200, 300, 300 );

	// Entirely outside
	if ( ! rect.IfOutside( Rect<int>( 50,50,60,60 ) ) ) goto Error;
	if ( ! rect.IfOutside( Rect<int>( -50,-50,-40,-40 ) ) ) goto Error;
	if ( ! rect.IfOutside( Rect<int>( 1000,1000,1100,1100 ) ) ) goto Error;

	// Overlaps.  IfOutside() should return false.
	if ( rect.IfOutside( Rect<int>(199, 199, 200, 200) ) ) goto Error;
	if ( rect.IfOutside( Rect<int>(1,1, 5000,5000) ) ) goto Error;
	if ( rect.IfOutside( Rect<int>(-500,-500, 5000,5000) ) ) goto Error;

	// Inside.  IfOutside() should return false.
	if ( rect.IfOutside( Rect<int>(200,200,201,201) ) ) goto Error;
	if ( rect.IfOutside( Rect<int>(201,201,299,299) ) ) goto Error;

    // Catch this stupid typo:
    //	 return     rect.x1 > x2	// if to the right
    //           || rect.x2 < y1	// if to the left
    //                        ^^ oops
    //           || rect.y1 > y2	// if below
    //           || rect.y2 < y1;	// if above
	Rect<int> rect2( 100,400, 200,500 );
	if ( rect2.IfOutside( Rect<int>(101,401, 199,499) ) ) goto Error;

	cerr << "ok\n";
 }

 {
	cerr << "----------- testing Grow() --------- \n";

	Rect<int> rect( 600, 601, 700, 701 );

	rect.Grow( 500, 501 );
	if (    rect.x1 != 500
         || rect.y1 != 501
         || rect.x2 != 700
         || rect.y2 != 701 ) goto Error;

	rect.Grow( 1010, 1011 );
	if (    rect.x1 != 500
         || rect.y1 != 501
         || rect.x2 != 1010
         || rect.y2 != 1011 ) goto Error;

	cerr << "ok\n";
 }

	return true;
Error:
	cerr << "** FAILED ** \n";
	return false;
}

/*****************************************************************************
 * Test Cube.
 *****************************************************************************/
bool
TestCube( void )
{
	cerr << "============================================\n";
	cerr << "Testing Cube class. \n";
	cerr << "============================================\n";

	// Unlike the Rect class, the Cube doesn't assume (x1,y1,z1) < (x2,y2,z2).

 {
	cerr << "----------- testing constructor, Set() --------- \n";
	// Test Set().  It shouldn't make any of x1,y1,z1 > any of x2,y2,z2.
	Cube<float> cube;
	cube.Set( 10, 11, 12,
              30, 31, 32 );
	if (    FNE( cube.x1, 10 )
         || FNE( cube.y1, 11 )
         || FNE( cube.z1, 12 )
         || FNE( cube.x2, 30 )
         || FNE( cube.y2, 31 )
         || FNE( cube.z2, 32 ) ) goto Error;
	cerr << "ok\n";
 }

 {
	cerr << "----------- testing IfInside() --------- \n";
	Cube<float> cube;
	cube.Set(  51,  52,  53,
              401, 402, 403 );
	// Should be inside:
	if ( ! cube.IfInside( 51, 52, 53 ) ) goto Error;
	if ( ! cube.IfInside( 100,100,100) ) goto Error;
	if ( ! cube.IfInside( 400,400,400) ) goto Error;
	if ( ! cube.IfInside( 401,402,403) ) goto Error;
	if ( ! cube.IfInside( WorldVertex(401,402,403) ) ) goto Error;
	// Should be outside:
	if ( cube.IfInside( -51, -52, -53 ) ) goto Error;
	if ( cube.IfInside( 50, 52, 53 ) ) goto Error;
	if ( cube.IfInside( WorldVertex(50, 52, 53) ) ) goto Error;

	cerr << "ok\n";
 }

 {
	cerr << "----------- testing IfInside(Posrad) --------- \n";
	Cube<float> cube( 100, 100, 100,
                      200, 200, 200 );

	// Cube::IfInside(Posrad) will return false if any part of Posrad
	// is outside this cube.

	// Posrad entirely inside:
	if ( ! cube.IfInside( Posrad(150,150,150,40) ) ) goto Error;
	if ( ! cube.IfInside( Posrad(110,110,110,2) ) ) goto Error;

	// Posrad partially inside, partially outside -- false expected.
	if ( cube.IfInside( Posrad(150,150,150,500) ) ) goto Error;
	if ( cube.IfInside( Posrad(-150,-150,-150,50000) ) ) goto Error;
	cerr << "ok\n";
 }

 {
	cerr << "----------- testing GetCenter() --------- \n";
	Cube<int> cube( 10, 10, 10,
                    20, 20, 20 );
	Position pos = cube.GetCenter();
	if ( FNE( pos.x, 15 ) ) goto Error;
	if ( FNE( pos.y, 15 ) ) goto Error;
	if ( FNE( pos.z, 15 ) ) goto Error;
	cerr << "ok\n";
 }

 {
	cerr << "----------- testing GetRadius() --------- \n";

	Cube<int> cube( 100, 100, 100,
                    200, 200, 200 );
	int radius = cube.GetRadius();
	if ( radius < 45 || radius > 65 ) goto Error;	// approximate

	Cube<int> cube2( 100, 500, 1000,
                     200, 500, 2000 );
	int radius2 = cube2.GetRadius();
	if ( radius2 < 450 || radius2 > 650 ) goto Error;	// approximate

	cerr << "ok\n";
 }

 {
	cerr << "----------- testing Grow() --------- \n";

	Cube<float> cube( 300, 300, 300,
                      400, 400, 400 );

	WorldVertex pos( 201, 202, 203 );

	if ( cube.IfInside( pos ) ) goto Error;	// not yet inside
	cube.Grow( pos );

	if ( ! cube.IfInside( pos ) ) goto Error;	// now should be inside

	if ( FNE(cube.x1, 201) ) goto Error;
	if ( FNE(cube.y1, 202) ) goto Error;
	if ( FNE(cube.z1, 203) ) goto Error;
	if ( FNE(cube.x2, 400) ) goto Error;
	if ( FNE(cube.y2, 400) ) goto Error;
	if ( FNE(cube.z2, 400) ) goto Error;

	cerr << "ok\n";
 }

 {
	cerr << "----------- testing Grow(Posrad) --------- \n";

	Cube<int> cube( 1000, 1000, 1000,
                    1010, 1010, 1010 );

	Posrad posrad( 990, 990, 990, 500 );

	if ( cube.IfInside( posrad ) ) goto Error;	// not yet inside
	cube.Grow( posrad );

	if ( ! cube.IfInside( posrad ) ) goto Error;	// now should be inside

	if ( cube.x1 > 600 ) goto Error;		// approx
	if ( cube.y1 > 600 ) goto Error;
	if ( cube.z1 > 600 ) goto Error;
	if ( cube.x2 < 1300 ) goto Error;
	if ( cube.y2 < 1300 ) goto Error;
	if ( cube.z2 < 1300 ) goto Error;

	cerr << "ok\n";
 }

	return true;
Error:
	cerr << "** FAILED ** \n";
	return false;
}

/*****************************************************************************
 * main.
 *****************************************************************************/
int main( int argc, char** argv )
{
	GFX_INIT( argv[0] );
	int res = 1;
	if (    TestArray()
         && TestVertex()
	     && TestPosrad()
	     && TestRect()
	     && TestCube() )
	{
		cerr << "Test PASSED.\n";
		res = 0;
	}
	else
	{
		cerr << "Test FAILED! \n";
		res = 1;
	}
	GFX_CLEANUP();
	exit( res );
}
