/*****************************************************************************
 * $Id: cache.cc,v 1.1 2006/10/21 00:19:08 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Test of Cache template class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "gfx_cache.hh"

class S
{
public:
			S( int x ) { val = x; }
			~S() { cerr << "~S() called (this=" << this << ")\n"; }
	int		val;
};

class Owner
{
	// nothing
	int unused;
};

/*****************************************************************************
 * Test.
 *****************************************************************************/
namespace gfx {
bool
CacheTest( void )
{
	//------------------------------------------------------------------------
	// Test caching items by different object owners (typical case).
	//------------------------------------------------------------------------

	Cache<S>* cache = new Cache<S>( 3, Cache<S>::SCALAR );
	cache->mLimit = 3; // ensure limit is 3
	if ( cache->mCnt != 0 ) Fail();

	S* s1 = new S( 123 );
	S* s2 = new S( 345 );
	S* s3 = new S( 678 );
	S* s4 = new S( 999 );

	S* t1 = NULL; // temp
	S* t2 = NULL; // temp
	S* t3 = NULL; // temp
//	S* t4 = NULL; // temp

	cerr << "s1 = " << (long)s1 << endl;
	cerr << "s2 = " << (long)s2 << endl;
	cerr << "s3 = " << (long)s3 << endl;
	cerr << "s4 = " << (long)s4 << endl;

	CacheOpq<S>	cacheOpq1;
	CacheOpq<S>	cacheOpq2;
	CacheOpq<S>	cacheOpq3;
	
	cerr << "Putting 3 items into cache. \n";
	cache->Put( &cacheOpq1, s1 );
	cache->Put( &cacheOpq2, s2 );
	cache->Put( &cacheOpq3, s3 );
	if ( cache->mCnt != 3 ) Fail();

	cerr << "Trying to get the same 3 items from cache...";
	t1 = cache->Get( &cacheOpq1 );
	t2 = cache->Get( &cacheOpq2 );
	t3 = cache->Get( &cacheOpq3 );
	if (    t1 == s1
         && t2 == s2
         && t3 == s3
         && t1->val == 123
         && t2->val == 345
         && t3->val == 678 )
		cerr << "OK.\n";
	else
		Fail();
	if ( cache->mCnt != 3 ) Fail(); // yes, 3, Get() doesn't remove

	//------------------------------------------------------------------------
	// Test adding a 4th item which should trigger an eviction.
	//------------------------------------------------------------------------

	cerr << "Testing adding 4th item into 3-element cache. \n";
	CacheOpq<S>	cacheOpq4;
	cache->Put( &cacheOpq4, s4 );
	cerr << "First item should be evicted, trying to get it...";
	if ( cache->Get( &cacheOpq1 ) == NULL )
		cerr << "OK\n(first item no longer in cache)\n";
	else
		Fail();
	cerr << "2nd/3rd/4th items should be cached, trying to get them...";
	if (    cache->Get( &cacheOpq2 ) == s2
         && cache->Get( &cacheOpq3 ) == s3
         && cache->Get( &cacheOpq4 ) == s4 )
		cerr << "OK\n";
	else
		Fail();

	//------------------------------------------------------------------------
	// Test adding a second set of 3 items/owners which should supplant the older ones.
	//------------------------------------------------------------------------

	S* b_s1 = new S( 321 );
	S* b_s2 = new S( 543 );
	S* b_s3 = new S( 876 );

	CacheOpq<S>	cacheOpq1_b;
	CacheOpq<S>	cacheOpq2_b;
	CacheOpq<S>	cacheOpq3_b;

	cerr << "b_s1 = " << (long)b_s1 << endl;
	cerr << "b_s2 = " << (long)b_s2 << endl;
	cerr << "b_s3 = " << (long)b_s3 << endl;

	cerr << "Putting a different set of 3 items into cache. \n";
	cache->Put( &cacheOpq1_b, b_s1 );
	if ( cache->mCnt != 3 ) Fail();
	cache->Put( &cacheOpq2_b, b_s2 );
	if ( cache->mCnt != 3 ) Fail();
	cache->Put( &cacheOpq3_b, b_s3 );
	if ( cache->mCnt != 3 ) Fail();

	return true;
}
} // namespace gfx

/*****************************************************************************
 * main.
 *****************************************************************************/
int
main( int argc, char** argv )
{
	GFX_INIT( argv[0] );
	int res = 1;
	if ( CacheTest() )
	{
		cerr << "Test PASSED. \n";
		res = 0;
	}
	else
	{
		cerr << "Test FAILED! \n";
		res = 1;
	}
	GFX_CLEANUP();
	exit( res );
}
